<?php

namespace Drupal\lb_theme_switcher\Theme;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Config\ImmutableConfig;
use Drupal\Core\Routing\AdminContext;
use Drupal\Core\Routing\RouteMatchInterface;
use Drupal\Core\Theme\ThemeNegotiatorInterface;
use Drupal\layout_builder\Plugin\SectionStorage\OverridesSectionStorage;
use Drupal\node\NodeInterface;

/**
 * Switch to Layout Builder-compatible theme when on LB pages.
 */
class LayoutBuilderThemeNegotiator implements ThemeNegotiatorInterface {

  /**
   * AdminContext service.
   *
   * @var \Drupal\Core\Routing\AdminContext
   */
  protected AdminContext $adminContext;

  /**
   * Theme switcher configuration object.
   *
   * @var \Drupal\Core\Config\ImmutableConfig
   */
  protected ImmutableConfig $config;

  /**
   * LayoutBuilderThemeNegotiator constructor.
   *
   * @param \Drupal\Core\Routing\AdminContext $admin_context
   *   AdminContext service.
   */
  public function __construct(
    AdminContext $admin_context,
    ConfigFactoryInterface $config_factory
  ) {
    $this->adminContext = $admin_context;
    $this->config = $config_factory->get('lb_theme_switcher.settings');
  }

  /**
   * {@inheritdoc}
   */
  public function applies(RouteMatchInterface $route_match) {
    if ($this->adminContext->isAdminRoute()) {
      return FALSE;
    }

    $node = $route_match->getParameter('node') ?? FALSE;
    $lb_used = FALSE;
    if ($node instanceof NodeInterface) {
      if ($node->hasField('field_use_layout_builder')) {
        $lb_used = (bool) $node->field_use_layout_builder->value;
      }
      elseif ($node->hasField(OverridesSectionStorage::FIELD_NAME)) {
        $lb_used = TRUE;
      }
    }

    if (
      (
        $node instanceof NodeInterface
        && $route_match->getRouteName() == 'entity.node.canonical'
        && $lb_used
      )
      || $route_match->getRouteName() == 'layout_builder.overrides.node.view'
      || $route_match->getRouteName() == 'layout_builder.defaults.node.view'
    ) {
      return TRUE;
    }

    return FALSE;
  }

  /**
   * {@inheritdoc}
   */
  public function determineActiveTheme(RouteMatchInterface $route_match) {
    return $this->config->get('theme_name');
  }

}
